import pytest

from seeq.base import gconfig
from seeq import spy

from . import test_common


@pytest.mark.system
def test_bad_login():
    try:
        spy.login('mark.derbecker@seeq.com', 'DataLab!', url='http://bogus')
        assert False, 'Exception was not raised with bogus url'
    except RuntimeError:
        pass

    # Remove overrides that resulted from spy.login() with bogus URL
    gconfig.override_global_property('seeq_server_hostname', '')
    gconfig.override_global_property('seeq_server_port', '')
    gconfig.override_global_property('seeq_secure_port', '')

    try:
        spy.login('mark.derbecker@seeq.com', 'DataLab!', auth_provider='bogus')
        assert False, 'Exception was not raised with bogus auth_provider'
    except RuntimeError:
        pass


@pytest.mark.system
def test_good_login():
    test_common.login()



import pytest

from seeq import spy

from . import test_common


@pytest.mark.system
def test_pull():
    test_common.login()

    search_results = spy.search({
        'Name': 'Area A_Temperature'
    })

    spy.pull(search_results, start='2019-01-01T00:00:00.000Z', end='2019-06-01T00:00:00.000Z')



import re

import pandas as pd

from seeq.sdk import *

from . import _common
from . import _login
from . import _push


def search(query, workbook=_common.DEFAULT_WORKBOOK_PATH, all_properties=False):
    """
    Issues a query to the Seeq Server to retrieve metadata for signals,
    conditions, scalars and assets. This metadata can be used to retrieve
    samples, capsules for a particular time range.
    :param query: A dictionary of property / match-criteria pairs. Match
    criteria uses the same syntax as the Data tab in Seeq.
    :type query: dict
    :param workbook: A path string (with >> delimiters) to indicate a workbook
    for which to scope the search to. Note that globally scoped items will
    still be returned.
    :type workbook: str
    :param all_properties: True if all item properties should be retrieved.
    This currently makes the search operation much slower as retrieval of
    properties for an item requires a separate call.
    :return: A DataFrame with rows for each item found and columns for each
    property.
    :rtype: pd.DataFrame
    """
    items_api = ItemsApi(_login.client)
    trees_api = TreesApi(_login.client)

    if isinstance(query, pd.DataFrame):
        queries = query.to_dict(orient='records')
        comparison = '=='
    else:
        queries = [query]
        comparison = '~='

    data = list()
    for query in queries:
        item_types = list()
        clauses = list()

        if _common.present(query, 'Type'):
            item_type_specs = list()
            if isinstance(query['Type'], list):
                item_type_specs.extend(query['Type'])
            else:
                item_type_specs.append(query['Type'])

            for item_type_spec in item_type_specs:
                if item_type_spec == 'Signal':
                    item_types.extend(['StoredSignal', 'CalculatedSignal'])
                elif item_type_spec == 'Condition':
                    item_types.extend(['StoredCondition', 'CalculatedCondition'])
                elif item_type_spec == 'Scalar':
                    item_types.extend(['StoredScalar', 'CalculatedScalar'])
                else:
                    item_types.append(item_type_spec)

            del query['Type']

        if _common.present(query, 'Datasource Name'):
            datasource_results = items_api.search_items(filters=['Name == %s' % _common.get(query, 'Datasource Name')],
                                                        types=['Datasource'],
                                                        limit=100000)  # type: ItemSearchPreviewPaginatedListV1

            if len(datasource_results.items) > 1:
                raise Exception('Multiple datasources found that match "%s"' % _common.get(query, 'Datasource Name'))
            elif len(datasource_results.items) == 1:
                datasource = datasource_results.items[0]  # type: ItemSearchPreviewV1
                query['Datasource ID'] = items_api.get_property(id=datasource.id, property_name='Datasource ID').value
                query['Datasource Class'] = items_api.get_property(id=datasource.id,
                                                                   property_name='Datasource Class').value

            del query['Datasource Name']

        if _common.present(query, 'Asset') and not _common.present(query, 'Path'):
            raise Exception('"Path" query parameter must be present when "Asset" parameter present')

        asset_id = None
        if _common.present(query, 'Path'):
            path = re.split(r'\s*>>\s*', query['Path'])
            if _common.present(query, 'Asset'):
                path.append(query['Asset'])
                del query['Asset']

            for part in path:
                if not asset_id:
                    tree_output = trees_api.get_tree_root_nodes()  # type: AssetTreeOutputV1
                else:
                    tree_output = trees_api.get_tree(id=asset_id)  # type: AssetTreeOutputV1

                found = False
                for child in tree_output.children:  # type: TreeItemOutputV1
                    if child.name == part:
                        found = True
                        asset_id = child.id
                        break

                if not found:
                    print('Could not found asset "%s"' % query['Path'])
                    return

            del query['Path']

        for prop_name in ['Name', 'Description', 'Datasource Class', 'Datasource ID', 'Data ID']:
            if prop_name in query and query[prop_name] is not None:
                clauses.append(prop_name + comparison + query[prop_name])
                del query[prop_name]

        filters = [' && '.join(clauses)] if len(clauses) > 0 else []

        kwargs = {
            'filters': filters,
            'types': item_types,
            'limit': _common.DEFAULT_SEARCH_PAGE_SIZE
        }

        if asset_id:
            kwargs['asset'] = asset_id

        if workbook:
            workbook_id = _push.reify_workbook(workbook, create=False)
            if workbook_id:
                kwargs['scope'] = workbook_id

        if 'Scoped To' in query and query['Scoped To'] is not None:
            kwargs['scope'] = query['Scoped To']
            kwargs['filters'].append('@excludeGloballyScoped')

        columns = list()

        def _do_search(offset):
            kwargs['offset'] = offset
            _common.ipython_clear_output(wait=True)
            print('Retrieved %d items...' % offset)
            return items_api.search_items(**kwargs)

        def _gather_results(result):
            prop_dict = dict()

            def _add_to_dict(key, val):
                prop_dict[key] = val

                # We want the columns to appear in a certain order (the order we added them in) for readability
                if key not in columns:
                    columns.append(key)

            _add_to_dict('ID', result.id)
            if len(result.ancestors) > 0:
                _add_to_dict('Path', ' >> '.join([a.name for a in result.ancestors[0:-1]]))
                _add_to_dict('Asset', result.ancestors[-1].name)
            _add_to_dict('Name', result.name)
            _add_to_dict('Description', result.description)
            _add_to_dict('Type', result.type)
            _add_to_dict('Value Unit Of Measure', result.value_unit_of_measure)
            if all_properties:
                item = items_api.get_item_and_all_properties(id=result.id)  # type: ItemOutputV1
                for prop in item.properties:  # type: PropertyOutputV1
                    _add_to_dict(prop.name, prop.value)

            for required_prop_name, required_prop_value in query.items():
                if required_prop_name not in prop_dict:
                    return

                if not re.match(required_prop_value, prop_dict[required_prop_name]):
                    return

            data.append(prop_dict)

        _iterate_over_output(_do_search, 'items', _gather_results)

        _common.ipython_clear_output()

        return pd.DataFrame(data=data, columns=columns)


def _iterate_over_output(output_func, collection_name, action_func):
    offset = 0
    while True:
        output = output_func(offset)

        collection = getattr(output, collection_name)

        for item in collection:
            action_func(item)

        if len(collection) != output.limit:
            break

        offset += output.limit



import pandas as pd

from .. import _common


def instantiate(model_module, metadata):
    return pd.DataFrame(_instantiate(model_module, metadata))


def _instantiate(model_module, metadata):
    results = list()
    unique_assets = metadata[['Path', 'Asset', 'Template']].drop_duplicates().dropna()
    for index, row in unique_assets.iterrows():
        if not _common.present(row, 'Template') or not _common.present(row, 'Asset') or not _common.present(row, 'Path'):
            continue

        template = getattr(model_module, row['Template'].replace(' ', '_'))
        instance = template({
            'Name': row['Asset'],
            'Asset': row['Asset'],
            'Path': row['Path']
        })

        instance_metadata = metadata[(metadata['Asset'] == row['Asset']) & (metadata['Path'] == row['Path'])]

        results.extend(instance.build(instance_metadata))

    return results



# coding: utf-8

"""
    Seeq REST API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 0.44.00-SNAPSHOT
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


from pprint import pformat
from six import iteritems
import re


class ItemPreviewWithAssetsV1(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """


    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'ancestors': 'list[ItemPreviewV1]',
        'href': 'str',
        'id': 'str',
        'is_archived': 'bool',
        'name': 'str',
        'type': 'str'
    }

    attribute_map = {
        'ancestors': 'ancestors',
        'href': 'href',
        'id': 'id',
        'is_archived': 'isArchived',
        'name': 'name',
        'type': 'type'
    }

    def __init__(self, ancestors=None, href=None, id=None, is_archived=False, name=None, type=None):
        """
        ItemPreviewWithAssetsV1 - a model defined in Swagger
        """

        self._ancestors = None
        self._href = None
        self._id = None
        self._is_archived = None
        self._name = None
        self._type = None

        if ancestors is not None:
          self.ancestors = ancestors
        if href is not None:
          self.href = href
        if id is not None:
          self.id = id
        if is_archived is not None:
          self.is_archived = is_archived
        if name is not None:
          self.name = name
        if type is not None:
          self.type = type

    @property
    def ancestors(self):
        """
        Gets the ancestors of this ItemPreviewWithAssetsV1.
        The list of ancestors in the asset tree, ordered with the root ancestor first, if the item is in an asset tree. If an item is in more than one asset tree an arbitrary one will be chosen.

        :return: The ancestors of this ItemPreviewWithAssetsV1.
        :rtype: list[ItemPreviewV1]
        """
        return self._ancestors

    @ancestors.setter
    def ancestors(self, ancestors):
        """
        Sets the ancestors of this ItemPreviewWithAssetsV1.
        The list of ancestors in the asset tree, ordered with the root ancestor first, if the item is in an asset tree. If an item is in more than one asset tree an arbitrary one will be chosen.

        :param ancestors: The ancestors of this ItemPreviewWithAssetsV1.
        :type: list[ItemPreviewV1]
        """

        self._ancestors = ancestors

    @property
    def href(self):
        """
        Gets the href of this ItemPreviewWithAssetsV1.
        The href that can be used to interact with the item

        :return: The href of this ItemPreviewWithAssetsV1.
        :rtype: str
        """
        return self._href

    @href.setter
    def href(self, href):
        """
        Sets the href of this ItemPreviewWithAssetsV1.
        The href that can be used to interact with the item

        :param href: The href of this ItemPreviewWithAssetsV1.
        :type: str
        """
        if href is None:
            raise ValueError("Invalid value for `href`, must not be `None`")

        self._href = href

    @property
    def id(self):
        """
        Gets the id of this ItemPreviewWithAssetsV1.
        The ID that can be used to interact with the item

        :return: The id of this ItemPreviewWithAssetsV1.
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Sets the id of this ItemPreviewWithAssetsV1.
        The ID that can be used to interact with the item

        :param id: The id of this ItemPreviewWithAssetsV1.
        :type: str
        """
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")

        self._id = id

    @property
    def is_archived(self):
        """
        Gets the is_archived of this ItemPreviewWithAssetsV1.
        Whether item is isArchived

        :return: The is_archived of this ItemPreviewWithAssetsV1.
        :rtype: bool
        """
        return self._is_archived

    @is_archived.setter
    def is_archived(self, is_archived):
        """
        Sets the is_archived of this ItemPreviewWithAssetsV1.
        Whether item is isArchived

        :param is_archived: The is_archived of this ItemPreviewWithAssetsV1.
        :type: bool
        """

        self._is_archived = is_archived

    @property
    def name(self):
        """
        Gets the name of this ItemPreviewWithAssetsV1.
        The human readable name

        :return: The name of this ItemPreviewWithAssetsV1.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Sets the name of this ItemPreviewWithAssetsV1.
        The human readable name

        :param name: The name of this ItemPreviewWithAssetsV1.
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")

        self._name = name

    @property
    def type(self):
        """
        Gets the type of this ItemPreviewWithAssetsV1.
        The type of the item

        :return: The type of this ItemPreviewWithAssetsV1.
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """
        Sets the type of this ItemPreviewWithAssetsV1.
        The type of the item

        :param type: The type of this ItemPreviewWithAssetsV1.
        :type: str
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")

        self._type = type

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        if not isinstance(other, ItemPreviewWithAssetsV1):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other
